' ***************************************************************************

' DIODNOTE.BAS

' This QuickBasic program executes Forward Voltage, Breakdown Voltage, and

' Leakage Current tests on 1N914 switching diodes (Keithley part no. RF-28).

' An output report showing test voltages, currents, and status is produced

' at the end of the tube or reel.  This program as it is written can test

' up to 120 diodes per execution.

' ***************************************************************************



' Declare subroutines

  DECLARE SUB SetupVfMemory (memloc%)

  DECLARE SUB SetupVbrkMemory (memloc%)

  DECLARE SUB SetupIlkMemory (memloc%)

  DECLARE SUB FormatReport ()



' Include IEEE488.2 library

' $INCLUDE: 'ieeeqb.bi'



' Initialize global constants, variables, and arrays

  Const NUMDIODES = 5                 'Number of diodes in tube or reel

  Const GOOD = 3                      'Pass pattern sent to handler

  Const QA = 5                        'Marginal failure pattern sent to handler

  Const BAD = 6                       'Failure pattern sent to handler

  Const ENTERKEY = 13                 'Ascii code for ENTER key

        

  COMMON SHARED AsciiBuffer$          'String variable

  COMMON SHARED addr%                 'Integer variable

  addr% = 24                          'Address of 2400

  AsciiBuffer$ = Space$(16 * 1024)    'Allocated space for 2400 data

   

  Dim Datalist$(1 To (3 * 3 * NUMDIODES))             'String array

  Dim Volts!(1 To (3 * NUMDIODES))                    'Single precision array

  Dim Current!(1 To (3 * NUMDIODES))                  'Single precision array

  Dim Result$(1 To (3 * NUMDIODES))                   'String array

    

' Initialize GPIB

  Call Initialize(21, 0)



' Define and save Forward Voltage, Breakdown Voltage and Leakage Current

' tests in source memory locations 1, 2, and 3, respectively

  Call SetupVfMemory(1)

  Call SetupVbrkMemory(2)

  Call SetupIlkMemory(3)



' Reset 2400 settings

  Call send(addr%, "*RST", Status%)

    

' Set up 2400 to execute three source memory configurations

  Call send(addr%, ":SOUR:FUNC       MEM", Status%)

  Call send(addr%, ":SOUR:MEM:POIN   3", Status%)



' Define 2400 Trigger Model:

' ARM counter limit equals number of diodes in tube or reel

' ARM event detector looks for Start of Test signal (currently set to IMM

' to enable testing without handler interface connected; change to STEST

' when handler input is available)

' TRIGGER counter limit equals number of tests to be run on each diode

  Call send(addr%, ":ARM:COUNT " + Str$(NUMDIODES), Status%)

  Call send(addr%, ":ARM:SOUR    IMM", Status%)

  Call send(addr%, ":TRIG:COUNT      3", Status%)

  Call send(addr%, ":TRIG:DELAY   0.005", Status%)



' Define which data elements are to be sent back to PC

  Call send(addr%, ":FORM:ELEM       VOLT, CURR, STATUS", Status%)



' Define overall pass code to be sent to handler for good diodes

' and command 2400 to send binning information upon completion of all

' source memory tests for that diode

  Call send(addr%, ":CALC2:CLIM:PASS:SOUR2 " + Str$(GOOD), Status%)

  Call send(addr%, ":CALC2:CLIM:BCON    END", Status%)



' Use fastest line cycle setting for increased speed

  Call send(addr%, ":SENS:VOLT:NPLC 0.1", Status%)

  Call send(addr%, ":SENS:CURR:NPLC 0.1", Status%)



' Prompt operator to indicate when tube or reel is loaded in handler

' Wait until ENTER is pressed to continue

  Print "Load tube or reel of diodes and press ENTER to continue"

  Do Until (INKEY$ = Chr$(ENTERKEY))

  Loop



' Turn 2400 source output on

  Call send(addr%, ":OUTPUT  ON", Status%)

   

' Initiate 2400 operation (bring instrument out of idle state) and download

' test data to PC

  Call send(addr%, ":READ?", Status%)

  Call enter(AsciiBuffer$, length%, addr%, Status%)



' Turn 2400 source output off

  Call send(addr%, ":OUTPUT  OFF", Status%)



' Print output report on screen and write report to diodout.txt.

' Report includes voltage, current, and status for each test.

  Call FormatReport

  Open "DIODOUT.TXT" For Append As #1                 'Open output file

  test$(1) = "VF"

  test$(2) = "VR"

  test$(3) = "IR"

  Print "Test", "Voltage", "Current", "Status"        'Print to screen

  Print #1, "Test", "Voltage", "Current", "Status"    'Print to file

  i = 1

  k = 1

  Do While (i <= (3 * 3 * NUMDIODES))

     For j = 1 To 3

       Print test$(j), Datalist$(i), Datalist$(i + 1), Result$(k)

       Print #1, test$(j), Datalist$(i), Datalist$(i + 1), Result$(k)

       i = i + 3

       k = k + 1

     Next j

     Print

     Print #1,

  Loop



  End



'**************************************************************************

Sub SetupVfMemory(memloc%)

' This subroutine defines the 2400 settings for the Forward Voltage Test

' and saves this configuration in the source memory location defined by

' memloc%

   

' Reset 2400 setttings

  Call send(addr%, "*RST", Status%)

   

' Source 10 mA of current with a 1mS settling time

  Call send(addr%, ":SOUR:FUNC             CURR", Status%)

  Call send(addr%, ":SOUR:CURR:RANG        10e-3", Status%)

  Call send(addr%, ":SOUR:CURR             10e-3", Status%)

  Call send(addr%, ":SOUR:DELAY            1e-3", Status%)



' Measure voltage in the 2 volt range with a compliance of 1.0 volts

  Call send(addr%, ":SENS:FUNC:CONC        OFF", Status%)

  Call send(addr%, ":SENS:FUNC             'VOLT'", Status%)

  Call send(addr%, ":SENS:VOLT:DC:RANG     2", Status%)

  Call send(addr%, ":SENS:VOLT:PROT        1", Status%)



' Make limit comparisons on voltage readings

  Call send(addr%, ":CALC2:FEED            VOLT", Status%)



' Define Limit 1: send BAD code if measurement is in compliance (>1V)

  Call send(addr%, ":CALC2:LIM1:COMP:FAIL  IN", Status%)

  Call send(addr%, ":CALC2:LIM1:COMP:SOUR2" + Str$(BAD), Status%)

  Call send(addr%, ":CALC2:LIM1:STAT       ON", Status%)



' Define Limit 2: send BAD code if measurement is >1V or <0.5V

  Call send(addr%, ":CALC2:LIM2:UPP        1", Status%)

  Call send(addr%, ":CALC2:LIM2:UPP:SOUR2" + Str$(BAD), Status%)

  Call send(addr%, ":CALC2:LIM2:LOW        0.5", Status%)

  Call send(addr%, ":CALC2:LIM2:LOW:SOUR2" + Str$(BAD), Status%)

  Call send(addr%, ":CALC2:LIM2:STAT       ON", Status%)



' Define Limit 3: send QA code if measurement is >0.8V or <0.7V

  Call send(addr%, ":CALC2:LIM3:UPP        0.8", Status%)

  Call send(addr%, ":CALC2:LIM3:UPP:SOUR2" + Str$(QA), Status%)

  Call send(addr%, ":CALC2:LIM3:LOW        0.7", Status%)

  Call send(addr%, ":CALC2:LIM3:LOW:SOUR2" + Str$(QA), Status%)

  Call send(addr%, ":CALC2:LIM3:STAT       ON", Status%)



' Send GOOD code if measurement is between 0.7 and 0.8 volts

  Call send(addr%, ":CALC2:CLIM:PASS:SOUR2" + Str$(GOOD), Status%)

    

' Save Forward Voltage Test configuration in source memory location #1

  Call send(addr%, ":SOUR:MEM:SAVE " + Str$(memloc%), Status%)



End Sub



'**************************************************************************

Sub SetupVbrkMemory(memloc%)

' This subroutine defines the 2400 settings for the Breakdown Voltage Test

' and saves this configuration in the source memory location defined by

' memloc%

 

' Reset 2400 settings

  Call send(addr%, "*RST", Status%)

   

' Source -1mA of current with a 1mS settling time

  Call send(addr%, ":SOUR:FUNC             CURR", Status%)

  Call send(addr%, ":SOUR:CURR:RANG        1e-3", Status%)

  Call send(addr%, ":SOUR:CURR             -1e-3", Status%)

  Call send(addr%, ":SOUR:DELAY            1e-3", Status%)



' Measure voltage in the 200 volt range with a compliance of 100 volts

' (specified minimum breakdown voltage)

  Call send(addr%, ":SENS:FUNC:CONC        OFF", Status%)

  Call send(addr%, ":SENS:FUNC             'VOLT'", Status%)

  Call send(addr%, ":SENS:VOLT:PROT        100", Status%)

  Call send(addr%, ":SENS:VOLT:DC:RANG     200", Status%)



' Make limit comparisons on voltage readings

  Call send(addr%, ":CALC2:FEED            VOLT", Status%)



' Define Limit 1: send BAD code if measurement is out of compliance

' (> -100V)

  Call send(addr%, ":CALC2:LIM1:COMP:FAIL  OUT", Status%)

  Call send(addr%, ":CALC2:LIM1:COMP:SOUR2" + Str$(BAD), Status%)

  Call send(addr%, ":CALC2:LIM1:STAT       ON", Status%)



' Send GOOD code if measurement is in compliance

  Call send(addr%, ":CALC2:CLIM:PASS:SOUR2" + Str$(GOOD), Status%)

    

' Save Breakdown Voltage Test configuration in source memory location #2

  Call send(addr%, ":SOUR:MEM:SAVE " + Str$(memloc%), Status%)



End Sub



'**************************************************************************

Sub SetupIlkMemory(memloc%)

' This subroutine defines the 2400 settings for the Leakage Current Test and

' saves this configuration in the source memory location defined by memloc%

   

' Reset 2400 settings

  Call send(addr%, "*RST", Status%)

   

' Source -75 volts with a 10mS settling time

  Call send(addr%, ":SOUR:FUNC             VOLT", Status%)

  Call send(addr%, ":SOUR:VOLT:RANG        200", Status%)

  Call send(addr%, ":SOUR:VOLT             -75", Status%)

  Call send(addr%, ":SOUR:DELAY            10e-3", Status%)



' Measure current in the 10 microamp range with a compliance

' of 5 microamps (specified maximum leakage current)

  Call send(addr%, ":SENS:FUNC:CONC        OFF", Status%)

  Call send(addr%, ":SENS:FUNC             'CURR'", Status%)

  Call send(addr%, ":SENS:CURR:DC:RANG     10e-6", Status%)

  Call send(addr%, ":SENS:CURR:PROT        5e-6", Status%)



' Make limit comparisons on current reading

  Call send(addr%, ":CALC2:FEED            CURR", Status%)



' Define Limit 1: send BAD code if measurement is in compliance

' (< -5.0 microamps)

  Call send(addr%, ":CALC2:LIM1:COMP:FAIL  IN", Status%)

  Call send(addr%, ":CALC2:LIM1:COMP:SOUR2" + Str$(BAD), Status%)

  Call send(addr%, ":CALC2:LIM1:STAT       ON", Status%)



' Define Limit 2: send BAD code if measurement is < -5.0 microamps

' or > 0.0 (indicating incorrect polarity)

  Call send(addr%, ":CALC2:LIM2:UPP        0", Status%)

  Call send(addr%, ":CALC2:LIM2:UPP:SOUR2" + Str$(BAD), Status%)

  Call send(addr%, ":CALC2:LIM2:LOW        -5e-6", Status%)

  Call send(addr%, ":CALC2:LIM2:LOW:SOUR2" + Str$(BAD), Status%)

  Call send(addr%, ":CALC2:LIM2:STAT       ON", Status%)



' Define Limit 3: send QA code if measurement is < -1 microamp

  Call send(addr%, ":CALC2:LIM3:UPP        0", Status%)

  Call send(addr%, ":CALC2:LIM3:UPP:SOUR2" + Str$(QA), Status%)

  Call send(addr%, ":CALC2:LIM3:LOW        -1e-6", Status%)

  Call send(addr%, ":CALC2:LIM3:LOW:SOUR2" + Str$(QA), Status%)

  Call send(addr%, ":CALC2:LIM3:STAT       ON", Status%)



' Send GOOD code if measurement is > -1 microamp and < 0

  Call send(addr%, ":CALC2:CLIM:PASS:SOUR2" + Str$(GOOD), Status%)



' Save Leakage Current Test configuration in source memory location #3

  Call send(addr%, ":SOUR:MEM:SAVE " + Str$(memloc%), Status%)



End Sub



'**************************************************************************

Sub FormatReport()

' This subroutine formats the AsciiBuffer data string returned from the

' 2400 into arrays which can be printed or saved for numerical analysis.

' Data elements are separated by locating commas in the AsciiBuffer string.



' Initialize variables

  k = 1

  location2% = 0



  Do While (k < (NUMDIODES * 3 * 3))      'Repeat loop for each data entry

     location1% = location2%              'Previous comma location

     pointer% = location1% + 1            'Starting point for next search

     found% = 0

     Do While (found% = 0)                'Repeat loop until comma is found

        If Asc(Mid$(AsciiBuffer$, pointer%)) = 44 Then

           found% = 1

           location2% = pointer%          'New comma location

        Else

           pointer% = pointer% + 1

        End If

     Loop

  

     length% = location2% - location1% - 1      'Length of data element

     Datalist$(k) = Mid$(AsciiBuffer$, location1% + 1, length%)

     k = k + 1

  Loop



  location1% = location2%

  Datalist$(k) = Mid$(AsciiBuffer$, location1% + 1, 14)   'Last element



' Convert voltage and current data elements into single precision numbers,

' convert status element (the decimal equivalent of a 16 bit binary word)

' into "GOOD", "QA", or "BAD" string, depending on the values of bits 8

' and 9, and save each element type in a separate array

  k = 1

  For i = 1 To (NUMDIODES * 3)

     Volts!(i) = Val(Datalist$(k))

     Current!(i) = Val(Datalist$(k + 1))

  

     Digstatus& = Int(Val(Datalist$(k + 2)))

     Bittest% = Digstatus& And 768        'Isolate bits 8 and 9 in status word

     If Bittest% = 0 Then

        Result$(i) = "GOOD"               'Bits 8 and 9 = "00"

     Else

        If Bittest% = 768 Then

           Result$(i) = "QA"              'Bits 8 and 9 = "11"

        Else

           Result$(i) = "BAD"             'Bits 8 and 9 = "01" or "10"

        End If

     End If

  

     k = k + 3

  Next i



End Sub


