' ***************************************************************************
' FLIPPOL.BAS
' This QuickBasic program executes Forward Voltage, Breakdown Voltage, and
' Leakage Current tests on 1N914 switching diodes (Keithley part no. RF-28).
' An output report showing test voltages, currents, and status is produced
' at the end of the tube or reel.  This program as it is written can test
' up to 120 diodes per execution. This program is used with a handler that
' orients (flips) the device after it has been tested.  The SourceMeter will
' pass information to the handler via the digital I/O port.  Different I/O
' patterns are used to signal whether the part passed or failed and which
' orientation it was tested in (forward or reverse polarity).
' ***************************************************************************

' Declare subroutines
DECLARE SUB Polarity (memloc%)
DECLARE SUB SetupVfMemory (memloc%)
DECLARE SUB SetupVbrkMemory (memloc%)
DECLARE SUB SetupIlkMemory (memloc%)
DECLARE SUB SetupNegVfMemory (memloc%)
DECLARE SUB SetupNegVbrkMemory (memloc%)
DECLARE SUB SetupNegIlkMemory (memloc%)
DECLARE SUB FormatReport ()

' Include IEEE488.2 library
'$INCLUDE: 'ieeeqb.bi'

' Initialize global constants, variables, and arrays
CONST NUMDIODES = 4             'Number of diodes in tube or reel
CONST GOODF = 15                'Pass pattern sent to handler-forward polarity
CONST GOODR = 11                'Pass pattern sent to handler-reverse polarity
CONST QAF = 6                   'Marginal failure sent to handler-forward polarity
CONST QAR = 7                   'Marginal failure sent to handler-reverse polarity
CONST BADF = 8                  'Failure pattern sent to handler-forward polarity
CONST BADR = 9                  'Failure pattern sent to handler-reverse polarity
CONST POLF = 12                 'Part in handler-forward polarity
CONST POLR = 14                 'Part in handler-reverse polarity
CONST ENTERKEY = 13             'Ascii code for ENTER key
COMMON SHARED AsciiBuffer$      'String variable
COMMON SHARED addr%             'Integer variable
addr% = 24                      'Address of 2400

AsciiBuffer$ = SPACE$(16 * 1024)                 'Allocated space for 2400 data
DIM SHARED Datalist$(1 TO (4 * 3 * NUMDIODES))   'String array
DIM SHARED Volts!(1 TO (4 * NUMDIODES))          'Single precision array
DIM SHARED Current!(1 TO (4 * NUMDIODES))        'Single precision array
DIM SHARED Result$(1 TO (4 * NUMDIODES))         'String array

' Initialize GPIB
CALL initialize(21, 0)

' Define and save Forward Voltage, Breakdown Voltage and Leakage Current
' tests in source memory locations 20, 21, and 22, respectively.  The testing
' begins by checked the polarity of the diode in the handler.  If it is
' oriented in the forward direction the source memory will execute points
' 20, 21, and 22.  If the diode is reversed, locations 2, 3, and 4 will
' execute.  Locations 2, 3, and 4 are identical to 20, 21, and 22 except
' all of the source and limit values are opposite polarity.
CALL Polarity(1)
CALL SetupNegVfMemory(2)
CALL SetupNegVbrkMemory(3)
CALL SetupNegIlkMemory(4)
CALL SetupVfMemory(20)
CALL SetupVbrkMemory(21)
CALL SetupIlkMemory(22)

' Reset 2400 settings
CALL send(addr%, "*RST", status%)

' Set up 2400 to execute three source memory configurations
CALL send(addr%, ":SOUR:FUNC MEM", status%)
CALL send(addr%, ":SOUR:MEM:POIN 4", status%)
CALL send(addr%, ":SOUR:MEM:STAR 1", status%)

' Define 2400 Trigger Model:
' ARM counter limit equals number of diodes in tube or reel
' ARM event detector looks for Start of Test signal (currently set to IMM
' to enable testing without handler interface connected; change to STEST
' when handler input is available)
' TRIGGER counter limit equals number of tests to be run on each diode
CALL send(addr%, ":ARM:COUNT " + STR$(NUMDIODES), status%)
CALL send(addr%, ":ARM:SOUR IMM", status%)
CALL send(addr%, ":TRIG:COUNT 4", status%)
CALL send(addr%, ":TRIG:DELAY 0", status%)

' Define which data elements are to be sent back to PC
CALL send(addr%, ":FORM:ELEM VOLT, CURR, STATUS", status%)

' Define overall pass code to be sent to handler for good diodes
' and command 2400 to send binning information upon completion of all
' source memory tests for that diode
CALL send(addr%, ":CALC2:CLIM:PASS:SOUR2 " + STR$(GOOD), status%)
CALL send(addr%, ":CALC2:CLIM:BCON END", status%)

' Use fastest line cycle setting for increased speed
CALL send(addr%, ":SENS:VOLT:NPLC 0.1", status%)
CALL send(addr%, ":SENS:CURR:NPLC 0.1", status%)

' Prompt operator to indicate when tube or reel is loaded in handler
' Wait until ENTER is pressed to continue
PRINT "Load tube or reel of diodes and press ENTER to continue"
DO UNTIL (INKEY$ = CHR$(ENTERKEY))
LOOP

' Turn 2400 source output on
CALL send(addr%, ":OUTP:STAT ON", status%)

' Initiate 2400 operation (bring instrument out of idle state) and download
' test data to PC
CALL send(addr%, ":READ?", status%)
CALL enter(AsciiBuffer$, length%, addr%, status%)

' Turn 2400 source output off
CALL send(addr%, ":OUTP:STAT OFF", status%)

' Print output report on screen and write report to diodout.txt.
' Report includes voltage, current, and status for each test.
CALL FormatReport
OPEN "DIODOUT.TXT" FOR APPEND AS #1                 'Open output file
test$(1) = "Polarity"
test$(2) = "VF"
test$(3) = "VR"
test$(4) = "IR"
PRINT "Test", "Voltage", "Current", "Status"        'Print to screen
PRINT #1, "Test", "Voltage", "Current", "Status"    'Print to file

i = 1
k = 1
DO WHILE (i <= (3 * 4 * NUMDIODES))
FOR j = 1 TO 4
PRINT test$(j), Datalist$(i), Datalist$(i + 1), Result$(k)
PRINT #1, test$(j), Datalist$(i), Datalist$(i + 1), Result$(k)
i = i + 3
k = k + 1
NEXT j

PRINT
PRINT #1,
LOOP
END
'**************************************************************************
'************************** End Main Program    *******************
'**************************************************************************

'**************************************************************************
'**************************************************************************

'**************************************************************************
'**************************************************************************

'**************************************************************************
'**************************************************************************

SUB FormatReport
' This subroutine formats the AsciiBuffer data string returned from the
' 2400 into arrays which can be printed or saved for numerical analysis.
' Data elements are separated by locating commas in the AsciiBuffer string.
' Initialize variables

k = 1
location2% = 0
DO WHILE (k < (NUMDIODES * 3 * 4))   'Repeat loop for each data entry
location1% = location2%              'Previous comma location
pointer% = location1% + 1            'Starting point for next search
found% = 0
	DO WHILE (found% = 0)                'Repeat loop until comma is found
		IF ASC(MID$(AsciiBuffer$, pointer%)) = 44 THEN
		found% = 1
		location2% = pointer%                'New comma location
		ELSE
		pointer% = pointer% + 1
		END IF
	LOOP
length% = location2% - location1% - 1      'Length of data element
Datalist$(k) = MID$(AsciiBuffer$, location1% + 1, length%)
k = k + 1
LOOP

location1% = location2%
Datalist$(k) = MID$(AsciiBuffer$, location1% + 1, 14)   'Last element

' Convert voltage and current data elements into single precision numbers,
' convert status element (the decimal equivalent of a 16 bit binary word)
' into "GOOD", "QA", or "BAD" string, depending on the values of bits 8
' and 9, and save each element type in a separate array.  This section may
' report that the polarity passed without a part present.  The idea of the
' polarity test is to check if there is an "OPEN" condition which results
' from the diode being packwards or if there is not a part present.  Be
' aware of this case, because the following is just an example of how the
' status byte can be used to report measurement results.

k = 1
j = 4
DumBit = 0
FOR i = 1 TO (NUMDIODES * 4)
Volts!(i) = VAL(Datalist$(k))
Current!(i) = VAL(Datalist$(k + 1))
Digstatus& = INT(VAL(Datalist$(k + 2)))
Poltest% = Digstatus& AND 264      'Isolate bits 3 and 8 in status word
Bittest% = Digstatus& AND 768      'Isolate bits 8 and 9 in status word
flag = j / 4
	IF flag = 1 THEN               'Use all three bits for first test
		SELECT CASE Poltest%       'Display result of polarity test
			CASE 264
				Result$(i) = "DIODE BACKWARD"
				DumBit = 1
			CASE 0
				Result$(i) = "POLARITY PASSED"
			CASE 8, 256
				Result$(i) = "?"
		END SELECT
		j = 0
	ELSE
		SELECT CASE Bittest%        'Display results for functional tests
			CASE 0                  'Bits 8 and 9 = "00"
				IF DumBit = 1 THEN
					Result$(i) = "SKIPPED"
				ELSE
					Result$(i) = "GOOD"
				END IF
			CASE 256, 512           'Bits 8 and 9 = "01" or "10"
				Result$(i) = "BAD"
			CASE 768                'Bits 8 and 9 = "11"
				Result$(i) = "QA"
		END SELECT
	END IF
k = k + 3
j = j + 1
NEXT i
END SUB

'**************************************************************************
'**************************************************************************
SUB Polarity (memloc%)
' This subroutine is done first and check the polarity of the diode.  If the
' diode is reversed in the handler a digital output line will tell the
' handler to flip the part.  After this first test, the 2400 will then proceed
' with performing the other three diode tests.

' Reset 2400 default settings
CALL send(addr%, "*RST", status%)

' Configure source and compliance values similar to Reverse Voltage test
' so that the diode will not be damaged no matter the handler orientation
CALL send(addr%, ":SOUR:FUNC CURR", status%)
CALL send(addr%, ":SOUR:CURR:RANG 1e-3", status%)
CALL send(addr%, ":SOUR:CURR 1e-3", status%)
CALL send(addr%, ":SOUR:DEL 0", status%)

' The 2400 measures voltage and checks for compliance. The key for the
' source and sense operation is to make them as fast as possible.
CALL send(addr%, ":SENS:FUNC:CONC OFF", status%)
CALL send(addr%, ":SENS:FUNC 'VOLT'", status%)
CALL send(addr%, ":SENS:VOLT:PROT 21", status%)
CALL send(addr%, ":SENS:VOLT:RANG 21", status%)
CALL send(addr%, ":SENS:VOLT:NPLC 0.01", status%)
CALL send(addr%, ":SYST:AZER OFF", status%)

' Define polarity test limits.
CALL send(addr%, ":CALC2:FEED VOLT", status%)
CALL send(addr%, ":CALC2:LIM1:STAT ON", status%)
CALL send(addr%, ":CALC2:LIM1:COMP:FAIL IN", status%)
CALL send(addr%, ":CALC2:LIM1:COMP:SOUR2 " + STR$(POSR), status%)
CALL send(addr%, ":CALC2:CLIM:PASS:SML 20", status%)
CALL send(addr%, ":CALC2:CLIM:PASS:SOUR2 " + STR$(POSF), status%)
CALL send(addr%, ":SOUR:MEM:SAVE " + STR$(memloc%), status%)

END SUB

SUB SetupIlkMemory (memloc%)
' This subroutine defines the 2400 settings for the Leakage Current Test and
' saves this configuration in the source memory location defined by memloc%

' Reset 2400 settings
CALL send(addr%, "*RST", status%)

' Source -75 volts with a 10mS settling time
CALL send(addr%, ":SOUR:FUNC VOLT", status%)
CALL send(addr%, ":SOUR:VOLT:RANG 200", status%)
CALL send(addr%, ":SOUR:VOLT -75", status%)
CALL send(addr%, ":SOUR:DELAY 10e-3", status%)

' Measure current in the 10 microamp range with a compliance
' of 5 microamps (specified maximum leakage current)
CALL send(addr%, ":SENS:FUNC:CONC OFF", status%)
CALL send(addr%, ":SENS:FUNC 'CURR'", status%)
CALL send(addr%, ":SENS:CURR:DC:RANG 10e-6", status%)
CALL send(addr%, ":SENS:CURR:PROT 5e-6", status%)

' Make limit comparisons on current reading
CALL send(addr%, ":CALC2:FEED CURR", status%)

' Define Limit 1: send BAD code if measurement is in compliance
' (< -5.0 microamps)
CALL send(addr%, ":CALC2:LIM1:COMP:FAIL IN", status%)
CALL send(addr%, ":CALC2:LIM1:COMP:SOUR2 " + STR$(BAD), status%)
CALL send(addr%, ":CALC2:LIM1:STAT ON", status%)

' Define Limit 2: send BAD code if measurement is < -5.0 microamps
' or > 0.0 (indicating incorrect polarity)
CALL send(addr%, ":CALC2:LIM2:UPP 0", status%)
CALL send(addr%, ":CALC2:LIM2:UPP:SOUR2 " + STR$(BAD), status%)
CALL send(addr%, ":CALC2:LIM2:LOW -5e-6", status%)
CALL send(addr%, ":CALC2:LIM2:LOW:SOUR2 " + STR$(BAD), status%)
CALL send(addr%, ":CALC2:LIM2:STAT ON", status%)

' Define Limit 3: send QA code if measurement is < -1 microamp
CALL send(addr%, ":CALC2:LIM3:UPP 0", status%)
CALL send(addr%, ":CALC2:LIM3:UPP:SOUR2 " + STR$(QA), status%)
CALL send(addr%, ":CALC2:LIM3:LOW -1e-6", status%)
CALL send(addr%, ":CALC2:LIM3:LOW:SOUR2 " + STR$(QA), status%)
CALL send(addr%, ":CALC2:LIM3:STAT ON", status%)

' Send GOOD code if measurement is > -1 microamp and < 0
CALL send(addr%, ":CALC2:CLIM:PASS:SOUR2 " + STR$(GOOD), status%)

' Save Leakage Current Test configuration in source memory location #3
CALL send(addr%, ":SOUR:MEM:SAVE " + STR$(memloc%), status%)
END SUB

'*************************************************************************
'*************************************************************************
'
SUB SetupNegIlkMemory (memloc%)
' This subroutine defines the 2400 settings for the Leakage Current Test and
' saves this configuration in the source memory location defined by memloc%
' This sub will be performed if the diode is reverse in the handler.

' Reset 2400 settings
CALL send(addr%, "*RST", status%)

' Source -75 volts with a 10mS settling time
CALL send(addr%, ":SOUR:FUNC VOLT", status%)
CALL send(addr%, ":SOUR:VOLT:RANG 200", status%)
CALL send(addr%, ":SOUR:VOLT 75", status%)
CALL send(addr%, ":SOUR:DELAY 10e-3", status%)

' Measure current in the 10 microamp range with a compliance
' of 5 microamps (specified maximum leakage current)
CALL send(addr%, ":SENS:FUNC:CONC OFF", status%)
CALL send(addr%, ":SENS:FUNC 'CURR'", status%)
CALL send(addr%, ":SENS:CURR:DC:RANG 10e-6", status%)
CALL send(addr%, ":SENS:CURR:PROT 5e-6", status%)

' Make sure auto zero is enabled for increased accuracy
CALL send(addr%, ":SYST:AZER ON", status%)

' Make limit comparisons on current reading
CALL send(addr%, ":CALC2:FEED CURR", status%)

' Define Limit 1: send BAD code if measurement is in compliance
' (> 5.0 microamps)
CALL send(addr%, ":CALC2:LIM1:COMP:FAIL IN", status%)
CALL send(addr%, ":CALC2:LIM1:COMP:SOUR2 " + STR$(BADR), status%)
CALL send(addr%, ":CALC2:LIM1:STAT ON", status%)
CALL send(addr%, ":CALC2:CLIM:PASS:SML NEXT", status%)

' Define Limit 2: send BAD code if measurement is > 5.0 microamps
' or > 0.0 (indicating incorrect polarity)
CALL send(addr%, ":CALC2:LIM2:UPP 5e-6", status%)
CALL send(addr%, ":CALC2:LIM2:UPP:SOUR2 " + STR$(BADR), status%)
CALL send(addr%, ":CALC2:LIM2:LOW 0", status%)
CALL send(addr%, ":CALC2:LIM2:LOW:SOUR2 " + STR$(BADR), status%)
CALL send(addr%, ":CALC2:LIM2:STAT ON", status%)

' Define Limit 3: send QA code if measurement is > 1 microamp
CALL send(addr%, ":CALC2:LIM3:UPP 1e-6", status%)
CALL send(addr%, ":CALC2:LIM3:UPP:SOUR2 " + STR$(QAR), status%)
CALL send(addr%, ":CALC2:LIM3:LOW 0", status%)
CALL send(addr%, ":CALC2:LIM3:LOW:SOUR2 " + STR$(QAR), status%)
CALL send(addr%, ":CALC2:LIM3:STAT ON", status%)

' Send GOOD code if measurement is < 1 microamp and > 0
CALL send(addr%, ":CALC2:CLIM:PASS:SOUR2 " + STR$(GOODR), status%)

' Save Leakage Current Test configuration in source memory location #4
CALL send(addr%, ":SOUR:MEM:SAVE " + STR$(memloc%), status%)
END SUB

'***********************************************************************
'***********************************************************************
'
SUB SetupNegVbrkMemory (memloc%)
' This subroutine defines the 2400 settings for the Breakdown Voltage Test
' and saves this configuration in the source memory location defined by
' memloc%.  This SUB will be performed if the diode is reversed in the
' handler.

' Reset 2400 settings
CALL send(addr%, "*RST", status%)

' Source -1mA of current with a 1mS settling time
CALL send(addr%, ":SOUR:FUNC CURR", status%)
CALL send(addr%, ":SOUR:CURR:RANG 1e-3", status%)
CALL send(addr%, ":SOUR:CURR 1e-3", status%)
CALL send(addr%, ":SOUR:DELAY 1e-3", status%)

' Measure voltage in the 200 volt range with a compliance of 100 volts
' (specified minimum breakdown voltage)
CALL send(addr%, ":SENS:FUNC:CONC OFF", status%)
CALL send(addr%, ":SENS:FUNC 'VOLT'", status%)
CALL send(addr%, ":SENS:VOLT:PROT 100", status%)
CALL send(addr%, ":SENS:VOLT:DC:RANG 200", status%)

' Make sure auto zero is enabled for increased accuracy
CALL send(addr%, ":SYST:AZER ON", status%)

' Make limit comparisons on voltage readings
CALL send(addr%, ":CALC2:FEED VOLT", status%)

' Define Limit 1: send BAD code if measurement is out of compliance
' (> -100V)
CALL send(addr%, ":CALC2:LIM1:COMP:FAIL OUT", status%)
CALL send(addr%, ":CALC2:LIM1:COMP:SOUR2 " + STR$(BADR), status%)
CALL send(addr%, ":CALC2:LIM1:STAT ON", status%)
CALL send(addr%, ":CALC2:CLIM:PASS:SML NEXT", status%)

' Send GOOD code if measurement is in compliance
CALL send(addr%, ":CALC2:CLIM:PASS:SOUR2 " + STR$(GOODR), status%)

' Save Breakdown Voltage Test configuration in source memory location #3
CALL send(addr%, ":SOUR:MEM:SAVE " + STR$(memloc%), status%)
END SUB

'***********************************************************************
'***********************************************************************
'
SUB SetupNegVfMemory (memloc%)
' This subroutine defines the 2400 settings for the Forward Voltage Test
' and saves this configuration in the source memory location defined by
' memloc%.  This SUB will be performed if the diode is reversed in the
' handler.

' Reset 2400 setttings
CALL send(addr%, "*RST", status%)

' Source 10 mA of current with a 1mS settling time
CALL send(addr%, ":SOUR:FUNC CURR", status%)
CALL send(addr%, ":SOUR:CURR:RANG 10e-3", status%)
CALL send(addr%, ":SOUR:CURR -10e-3", status%)
CALL send(addr%, ":SOUR:DELAY 1e-3", status%)

' Measure voltage in the 2 volt range with a compliance of 1.0 volts
CALL send(addr%, ":SENS:FUNC:CONC OFF", status%)
CALL send(addr%, ":SENS:FUNC 'VOLT'", status%)
CALL send(addr%, ":SENS:VOLT:DC:RANG 2", status%)
CALL send(addr%, ":SENS:VOLT:PROT 1", status%)

' Make sure auto zero is enabled for increased accuracy
CALL send(addr%, ":SYST:AZER ON", status%)

' Make limit comparisons on voltage readings
CALL send(addr%, ":CALC2:FEED VOLT", status%)

' Define Limit 1: send BAD code if measurement is in compliance (>1V)
CALL send(addr%, ":CALC2:LIM1:COMP:FAIL IN", status%)
CALL send(addr%, ":CALC2:LIM1:COMP:SOUR2 " + STR$(BADR), status%)
CALL send(addr%, ":CALC2:LIM1:STAT ON", status%)
CALL send(addr%, ":CALC2:CLIM:PASS:SML NEXT", status%)

' Define Limit 2: send BAD code if measurement is <-1V or >-0.5V
CALL send(addr%, ":CALC2:LIM2:UPP -0.5", status%)
CALL send(addr%, ":CALC2:LIM2:UPP:SOUR2 " + STR$(BADR), status%)
CALL send(addr%, ":CALC2:LIM2:LOW -1", status%)
CALL send(addr%, ":CALC2:LIM2:LOW:SOUR2 " + STR$(BADR), status%)
CALL send(addr%, ":CALC2:LIM2:STAT ON", status%)

' Define Limit 3: send QA code if measurement is <-0.8V or >-0.7V
CALL send(addr%, ":CALC2:LIM3:UPP -0.7", status%)
CALL send(addr%, ":CALC2:LIM3:UPP:SOUR2 " + STR$(QAR), status%)
CALL send(addr%, ":CALC2:LIM3:LOW -0.8", status%)
CALL send(addr%, ":CALC2:LIM3:LOW:SOUR2 " + STR$(QAR), status%)
CALL send(addr%, ":CALC2:LIM3:STAT ON", status%)

' Send GOOD code if measurement is between -0.7 and -0.8 volts
CALL send(addr%, ":CALC2:CLIM:PASS:SOUR2 " + STR$(GOODR), status%)

' Save Forward Voltage Test configuration in source memory location #2
CALL send(addr%, ":SOUR:MEM:SAVE " + STR$(memloc%), status%)

END SUB

SUB SetupVbrkMemory (memloc%)
' This subroutine defines the 2400 settings for the Breakdown Voltage Test
' and saves this configuration in the source memory location defined by
' memloc%

' Reset 2400 settings
CALL send(addr%, "*RST", status%)

' Source -1mA of current with a 1mS settling time
CALL send(addr%, ":SOUR:FUNC CURR", status%)
CALL send(addr%, ":SOUR:CURR:RANG 1e-3", status%)
CALL send(addr%, ":SOUR:CURR -1e-3", status%)
CALL send(addr%, ":SOUR:DELAY 1e-3", status%)

' Measure voltage in the 200 volt range with a compliance of 100 volts
' (specified minimum breakdown voltage)
CALL send(addr%, ":SENS:FUNC:CONC OFF", status%)
CALL send(addr%, ":SENS:FUNC 'VOLT'", status%)
CALL send(addr%, ":SENS:VOLT:PROT 100", status%)
CALL send(addr%, ":SENS:VOLT:DC:RANG 200", status%)

' Make limit comparisons on voltage readings
CALL send(addr%, ":CALC2:FEED VOLT", status%)

' Define Limit 1: send BAD code if measurement is out of compliance
' (> -100V)
CALL send(addr%, ":CALC2:LIM1:COMP:FAIL OUT", status%)
CALL send(addr%, ":CALC2:LIM1:COMP:SOUR2 " + STR$(BAD), status%)
CALL send(addr%, ":CALC2:LIM1:STAT ON", status%)

' Send GOOD code if measurement is in compliance
CALL send(addr%, ":CALC2:CLIM:PASS:SOUR2 " + STR$(GOOD), status%)

' Save Breakdown Voltage Test configuration in source memory location #2
CALL send(addr%, ":SOUR:MEM:SAVE " + STR$(memloc%), status%)
END SUB

SUB SetupVfMemory (memloc%)
' This subroutine defines the 2400 settings for the Forward Voltage Test
' and saves this configuration in the source memory location defined by
' memloc%

' Reset 2400 setttings
CALL send(addr%, "*RST", status%)

' Source 10 mA of current with a 1mS settling time
CALL send(addr%, ":SOUR:FUNC CURR", status%)
CALL send(addr%, ":SOUR:CURR:RANG 10e-3", status%)
CALL send(addr%, ":SOUR:CURR 10e-3", status%)
CALL send(addr%, ":SOUR:DELAY 1e-3", status%)

' Measure voltage in the 2 volt range with a compliance of 1.0 volts
CALL send(addr%, ":SENS:FUNC:CONC OFF", status%)
CALL send(addr%, ":SENS:FUNC 'VOLT'", status%)
CALL send(addr%, ":SENS:VOLT:DC:RANG 2", status%)
CALL send(addr%, ":SENS:VOLT:PROT 1", status%)

' Make limit comparisons on voltage readings
CALL send(addr%, ":CALC2:FEED VOLT", status%)

' Define Limit 1: send BAD code if measurement is in compliance (>1V)
CALL send(addr%, ":CALC2:LIM1:COMP:FAIL IN", status%)
CALL send(addr%, ":CALC2:LIM1:COMP:SOUR2 " + STR$(BAD), status%)
CALL send(addr%, ":CALC2:LIM1:STAT ON", status%)

' Define Limit 2: send BAD code if measurement is >1V or <0.5V
CALL send(addr%, ":CALC2:LIM2:UPP 1", status%)
CALL send(addr%, ":CALC2:LIM2:UPP:SOUR2 " + STR$(BAD), status%)
CALL send(addr%, ":CALC2:LIM2:LOW 0.5", status%)
CALL send(addr%, ":CALC2:LIM2:LOW:SOUR2 " + STR$(BAD), status%)
CALL send(addr%, ":CALC2:LIM2:STAT ON", status%)

' Define Limit 3: send QA code if measurement is >0.8V or <0.7V
CALL send(addr%, ":CALC2:LIM3:UPP 0.8", status%)
CALL send(addr%, ":CALC2:LIM3:UPP:SOUR2 " + STR$(QA), status%)
CALL send(addr%, ":CALC2:LIM3:LOW 0.7", status%)
CALL send(addr%, ":CALC2:LIM3:LOW:SOUR2 " + STR$(QA), status%)
CALL send(addr%, ":CALC2:LIM3:STAT ON", status%)

' Send GOOD code if measurement is between 0.7 and 0.8 volts
CALL send(addr%, ":CALC2:CLIM:PASS:SOUR2 " + STR$(GOOD), status%)

' Save Forward Voltage Test configuration in source memory location #1
CALL send(addr%, ":SOUR:MEM:SAVE " + STR$(memloc%), status%)

END SUB

